﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class KeijzerFunctionSix : RegressionToyBenchmark {

    public KeijzerFunctionSix() {
      Name = "Keijzer 6 f(x) = (30 * x * z) / ((x - 10)  * y^2)";
      Description = "Paper: Improving Symbolic Regression with Interval Arithmetic and Linear Scaling" + Environment.NewLine
        + "Authors: Maarten Keijzer" + Environment.NewLine
        + "Function: f(x) = (30 * x * z) / ((x - 10)  * y^2)" + Environment.NewLine
        + "range(train): 1000 points x,z = rnd(-1, 1), y = rnd(1, 2)" + Environment.NewLine
        + "range(test): 10000 points x,z = rnd(-1, 1), y = rnd(1, 2)" + Environment.NewLine
        + "Function Set: x + y, x * y, 1/x, -x, sqrt(x)";
      targetVariable = "F";
      inputVariables = new List<string>() { "X", "Y", "Z" };
      trainingPartition = new IntRange(0, 1000);
      testPartition = new IntRange(1001, 11000);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x, y, z;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x = data[0][i];
        y = data[1][i];
        z = data[2][i];
        results.Add((30 * x * z) / ((x - 10) * Math.Pow(y, 2)));
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();
      DoubleRange rangeXZ = new DoubleRange(-1, 1);
      DoubleRange rangeY = new DoubleRange(1, 2);
      dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(testPartition.End, rangeXZ));
      dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(testPartition.End, rangeY));
      dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(testPartition.End, rangeXZ));

      return dataList;
    }
  }
}
