﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class KotanchekFunction : RegressionToyBenchmark {

    public KotanchekFunction() {
      Name = "Vladislavleva Kotanchek";
      Description = "Paper: Order of Nonlinearity as a Complexity Measure for Models Generated by Symbolic Regression via Pareto Genetic Programming " + Environment.NewLine
        + "Authors: Ekaterina J. Vladislavleva, Member, IEEE, Guido F. Smits, Member, IEEE, and Dick den Hertog" + Environment.NewLine
        + "Function: F1(X1, X2) = e^-(X1 - 1)^2 / (1.2 + (X2 -2.5)^2" + Environment.NewLine
        + "Training Data: 100 points X1, X2 = Rand(0.3, 4)" + Environment.NewLine
        + "Test Data: 2026 points (X1, X2) = (-0.2:0.1:4.2)" + Environment.NewLine
        + "Function Set: +, -, *, /, sqaure, x^real, x + real, x + real, e^x, e^-x";
      targetVariable = "Y";
      inputVariables = new List<string>() { "X1", "X2" };
      trainingPartition = new IntRange(0, 100);
      testPartition = new IntRange(101, 2125);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x1, x2;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x1 = data[0][i];
        x2 = data[1][i];
        results.Add(Math.Exp(-Math.Pow(x1 - 1, 2)) / (1.2 + Math.Pow(x2 - 2.5, 2)));
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();
      DoubleRange trainingRange = new DoubleRange(0.3, 4);
      for (int i = 0; i < InputVariable.Count; i++) {
        dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(TrainingPartition.End, trainingRange));
      }

      List<double> oneVariableTestData = RegressionBenchmark.GenerateSteps(new DoubleRange(-0.2, 4.2), 0.1);
      List<List<double>> testData = new List<List<double>>() { oneVariableTestData, oneVariableTestData };
      testData = RegressionBenchmark.GenerateAllCombinationsOfValuesInLists(testData);
      for (int i = 0; i < InputVariable.Count; i++) {
        dataList[i].AddRange(testData[i]);
      }

      return dataList;
    }
  }
}
