﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class RationalPolynomial : RegressionToyBenchmark {

    public RationalPolynomial() {
      Name = "Vladislavleva RatPol3D";
      Description = "Paper: Order of Nonlinearity as a Complexity Measure for Models Generated by Symbolic Regression via Pareto Genetic Programming " + Environment.NewLine
        + "Authors: Ekaterina J. Vladislavleva, Member, IEEE, Guido F. Smits, Member, IEEE, and Dick den Hertog" + Environment.NewLine
        + "Function: F5(X1, X2, X3) = 30 * ((X1 - 1) * (X3 -1)) / (X2^2 * (X1 - 10))" + Environment.NewLine
        + "Training Data: 300 points X1, X3 = Rand(0.05, 2), X2 = Rand(1, 2)" + Environment.NewLine
        + "Test Data: 2701 points X1, X3 = (-0.05:0.15:2.1), X2 = (0.95:0.1:2.05)" + Environment.NewLine
        + "Function Set: +, -, *, /, sqaure, x^real, x + real, x + real";
      targetVariable = "Y";
      inputVariables = new List<string>() { "X1", "X2", "X3" };
      trainingPartition = new IntRange(0, 300);
      testPartition = new IntRange(301, 3000);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x1, x2, x3;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x1 = data[0][i];
        x2 = data[1][i];
        x3 = data[2][i];
        results.Add(30 * ((x1 - 1) * (x3 - 1)) / (Math.Pow(x2, 2) * (x1 - 10)));
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();
      dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(300, new DoubleRange(0.05, 2)));
      dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(300, new DoubleRange(1, 2)));
      dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(300, new DoubleRange(0.05, 2)));

      List<List<double>> testData = new List<List<double>>() { 
        RegressionBenchmark.GenerateSteps(new DoubleRange(-0.05, 2.1), 0.15), 
        RegressionBenchmark.GenerateSteps(new DoubleRange( 0.95, 2.05), 0.1),
        RegressionBenchmark.GenerateSteps(new DoubleRange(-0.05, 2.1), 0.15)
      };

      testData = RegressionBenchmark.GenerateAllCombinationsOfValuesInLists(testData);

      for (int i = 0; i < InputVariable.Count; i++) {
        dataList[i].AddRange(testData[i]);
      }

      return dataList;
    }
  }
}
