﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class SalutowiczFunctionTwoDimensional : RegressionToyBenchmark {

    public SalutowiczFunctionTwoDimensional() {
      Name = "Vladislavleva Salutowicz2D";
      Description = "Paper: Order of Nonlinearity as a Complexity Measure for Models Generated by Symbolic Regression via Pareto Genetic Programming " + Environment.NewLine
        + "Authors: Ekaterina J. Vladislavleva, Member, IEEE, Guido F. Smits, Member, IEEE, and Dick den Hertog" + Environment.NewLine
        + "Function: F3(X1, X2) = e^-X1 * X1^3 * cos(X1) * sin(X1) * (cos(X1)sin(X1)^2 - 1)(X2 - 5)" + Environment.NewLine
        + "Training Data: 601 points X1 = (0.05:0.1:10), X2 = (0.05:2:10.05)" + Environment.NewLine
        + "Test Data: 2554 points X1 = (-0.5:0.05:10.5), X2 = (-0.5:0.5:10.5)" + Environment.NewLine
        + "Function Set: +, -, *, /, sqaure, x^real, x + real, x + real, e^x, e^-x, sin(x), cos(x)" + Environment.NewLine + Environment.NewLine
        + "Important: The stepwidth of the variable X1 in the test partition has been set to 0.1, to fit the amount of data points.";
      targetVariable = "Y";
      inputVariables = new List<string>() { "X1", "X2" };
      trainingPartition = new IntRange(0, 601);
      testPartition = new IntRange(602, 3155);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x1, x2;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x1 = data[0][i];
        x2 = data[1][i];
        results.Add(Math.Exp(-x1) * Math.Pow(x1, 3) * Math.Cos(x1) * Math.Sin(x1) * (Math.Cos(x1) * Math.Pow(Math.Sin(x1), 2) - 1) * (x2 - 5));
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();
      List<List<double>> trainingData = new List<List<double>>() {
        RegressionBenchmark.GenerateSteps(new DoubleRange(0.05, 10), 0.1),
        RegressionBenchmark.GenerateSteps(new DoubleRange(0.05, 10.05), 2)
      };

      List<List<double>> testData = new List<List<double>>() {
        RegressionBenchmark.GenerateSteps(new DoubleRange(-0.5, 10.5), 0.1),
        RegressionBenchmark.GenerateSteps(new DoubleRange(-0.5, 10.5), 0.5)
      };

      trainingData = RegressionBenchmark.GenerateAllCombinationsOfValuesInLists(trainingData);
      testData = RegressionBenchmark.GenerateAllCombinationsOfValuesInLists(testData);

      for (int i = 0; i < InputVariable.Count; i++) {
        dataList.Add(trainingData[i]);
        dataList[i].AddRange(testData[i]);
      }

      return dataList;
    }
  }
}
