﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System;
using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class UnwrappedBallFunctionFiveDimensional : RegressionToyBenchmark {

    public UnwrappedBallFunctionFiveDimensional() {
      Name = "Vladislavleva UBall5D";
      Description = "Paper: Order of Nonlinearity as a Complexity Measure for Models Generated by Symbolic Regression via Pareto Genetic Programming " + Environment.NewLine
        + "Authors: Ekaterina J. Vladislavleva, Member, IEEE, Guido F. Smits, Member, IEEE, and Dick den Hertog" + Environment.NewLine
        + "Function: F4(X1, X2, X3, X4, X5) = 10 / (5 + Sum(Xi - 3)^2)" + Environment.NewLine
        + "Training Data: 1024 points Xi = Rand(0.05, 6.05)" + Environment.NewLine
        + "Test Data: 5000 points Xi = Rand(-0.25, 6.35)" + Environment.NewLine
        + "Function Set: +, -, *, /, sqaure, x^real, x + real, x + real";
      targetVariable = "Y";
      inputVariables = new List<string>() { "X1", "X2", "X3", "X4", "X5" };
      trainingPartition = new IntRange(0, 1024);
      testPartition = new IntRange(1025, 6024);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x1, x2, x3, x4, x5;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x1 = data[0][i];
        x2 = data[1][i];
        x3 = data[2][i];
        x4 = data[3][i];
        x5 = data[4][i];
        results.Add(10 / (5 + Math.Pow(x1 - 3, 2) + Math.Pow(x2 - 3, 2) + Math.Pow(x3 - 3, 2) + Math.Pow(x4 - 3, 2) + Math.Pow(x5 - 3, 2)));
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();
      DoubleRange testRange = new DoubleRange(0.05, 6.05);
      DoubleRange trainingRange = new DoubleRange(-0.25, 6.35);
      for (int i = 0; i < InputVariable.Count; i++) {
        dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(1024, testRange));
        dataList[i].AddRange(RegressionBenchmark.GenerateUniformDistributedValues(5000, trainingRange));
      }

      return dataList;
    }
  }
}
